@@ -6,7 +6,7 @@ use base 'DBIx::Class::Schema::Loader::DBI';
 use Carp::Clan qw/^DBIx::Class/;
 use Class::C3;
 
-our $VERSION = '0.04999_12';
+our $VERSION = '0.04999_13';
 
 =head1 NAME
 
@@ -122,6 +122,53 @@ sub _column_comment {
     $column_number );
 }
 
+# Make sure data_type's that don't need it don't have a 'size' column_info, and
+# set the correct precision for datetime and varbit types.
+sub _columns_info_for {
+    my $self = shift;
+    my ($table) = @_;
+
+    my $result = $self->next::method(@_);
+
+    foreach my $col (keys %$result) {
+        my $data_type = $result->{$col}{data_type};
+
+        # these types are fixed size
+        if ($data_type =~
+/^(?:bigint|int8|bigserial|serial8|bit|boolean|bool|box|bytea|cidr|circle|date|double precision|float8|inet|integer|int|int4|line|lseg|macaddr|money|path|point|polygon|real|float4|smallint|int2|serial|serial4|text)\z/i) {
+            delete $result->{$col}{size};
+        }
+# for datetime types, check if it has a precision or not
+        elsif ($data_type =~ /^(?:interval|time|timestamp)\b/) {
+            my ($precision) = $self->schema->storage->dbh
+                ->selectrow_array(<<EOF, {}, $table, $col);
+SELECT datetime_precision
+FROM information_schema.columns
+WHERE table_name = ? and column_name = ?
+EOF
+
+            if ((not $precision) || $precision !~ /^\d/) {
+                delete $result->{$col}{size};
+            }
+            else {
+                $result->{$col}{size} = $precision;
+            }
+        }
+        elsif ($data_type =~ /^(?:bit varying|varbit)\z/i) {
+            my ($precision) = $self->schema->storage->dbh
+                ->selectrow_array(<<EOF, {}, $table, $col);
+SELECT character_maximum_length
+FROM information_schema.columns
+WHERE table_name = ? and column_name = ?
+EOF
+
+            $result->{$col}{size} = $precision;
+        }
+    }
+
+    return $result;
+}
+
 sub _extra_column_info {
     my ($self, $info) = @_;
     my %extra_info;
@@ -140,7 +187,7 @@ L<DBIx::Class::Schema::Loader::DBI>
 
 =head1 AUTHOR
 
-See L<DBIx::Class::Schema::Loader/CONTRIBUTORS>.
+See L<DBIx::Class::Schema::Loader/AUTHOR> and L<DBIx::Class::Schema::Loader/CONTRIBUTORS>.
 
 =head1 LICENSE
 
